@@ -75,6 +75,7 @@ struct DNSRequest {
 	struct List znode;	/* DNSZone->host_list */
 
 	struct DNSContext *ctx;
+	struct DNSZone *zone;
 
 	struct List ucb_list;	/* DNSToken->node */
 
@@ -265,7 +266,7 @@ static void impl_launch_query(struct DNSRequest *req)
 	static const struct addrinfo hints = { .ai_socktype = SOCK_STREAM };
 
 	struct GaiContext *gctx = req->ctx->edns;
-	struct GaiRequest *grq = calloc(1, sizeof(*grq));
+	struct GaiRequest *grq;
 	int res;
 	struct gaicb *cb;
 
@@ -286,7 +287,11 @@ static void impl_launch_query(struct DNSRequest *req)
 	return;
 
 failed:
-	log_warning("dns: getaddrinfo_a(%s)=%d", req->name, res);
+	if (res == EAI_SYSTEM)
+		log_warning("dns: getaddrinfo_a(%s)=%d, errno=%d (%s)",
+			    req->name, res, errno, strerror(errno));
+	else
+		log_warning("dns: getaddrinfo_a(%s)=%d", req->name, res);
 	list_del(&grq->node);
 	free(grq);
 failed2:
@@ -471,6 +476,7 @@ static void udns_result_a4(struct dns_ctx *ctx, struct dns_rr_a4 *a4, void *data
 	if (a4) {
 		log_noise("udns_result_a4: %s: %d ips", req->name, a4->dnsa4_nrr);
 		res = convert_ipv4_result(a4->dnsa4_addr, a4->dnsa4_nrr);
+		free(a4);
 	}
 	got_result_gai(0, res, req);
 }
@@ -649,6 +655,8 @@ static void udns_result_soa(struct dns_ctx *uctx, struct SOA *soa, void *data)
 		  soa->dnssoa_retry, soa->dnssoa_expire, soa->dnssoa_minttl);
 
 	got_zone_serial(ctx, &soa->dnssoa_serial);
+
+	free(soa);
 }
 
 static int impl_query_soa_serial(struct DNSContext *ctx, const char *zonename)
@@ -740,7 +748,8 @@ static void req_free(struct AANode *node, void *arg)
 		freeaddrinfo(req->oldres);
 		req->oldres = NULL;
 	}
-	list_del(&req->znode);
+	if (req->zone)
+		statlist_remove(&req->zone->host_list, &req->znode);
 	free(req->name);
 	free(req);
 }
@@ -967,6 +976,7 @@ static void zone_register(struct DNSContext *ctx, struct DNSRequest *req)
 	if (n) {
 		/* already exists */
 		z = container_of(n, struct DNSZone, tnode);
+		req->zone = z;
 		statlist_append(&z->host_list, &req->znode);
 		return;
 	}
@@ -987,6 +997,7 @@ static void zone_register(struct DNSContext *ctx, struct DNSRequest *req)
 	aatree_insert(&ctx->zone_tree, (uintptr_t)z->zonename, &z->tnode);
 	list_append(&ctx->zone_list, &z->lnode);
 	statlist_append(&z->host_list, &req->znode);
+	req->zone = z;
 }
 
 static void zone_timer(int fd, short flg, void *arg)
@@ -1004,6 +1015,7 @@ static void zone_timer(int fd, short flg, void *arg)
 	z = container_of(el, struct DNSZone, lnode);
 	ctx->zone_state = 1;
 	ctx->cur_zone = z;
+	ctx->active++;
 	impl_query_soa_serial(ctx, z->zonename);
 }
 
